@@ -11,10 +11,38 @@ use std::io;
 
 use crate::blockchain::{Address, Fee, FeeStrategy, Onchain, Timelock};
 use crate::consensus::{self, CanonicalBytes, Decodable, Encodable};
-use crate::crypto::{Keys, SharedKeyId, SharedPrivateKeys, Signatures, TaggedElement};
+use crate::crypto::{Keys, SharedKeyId, SharedSecretKeys, Signatures, TaggedElement};
 use crate::protocol_message;
 use crate::swap::Swap;
 
+#[derive(Debug, Clone, Display)]
+#[display(Debug)]
+pub struct Proof<Ctx: Swap> {
+    pub proof: Ctx::Proof,
+}
+
+impl<Ctx> Encodable for Proof<Ctx>
+where
+    Ctx: Swap,
+{
+    fn consensus_encode<W: io::Write>(&self, s: &mut W) -> Result<usize, io::Error> {
+        self.proof.as_canonical_bytes().consensus_encode(s)
+    }
+}
+
+impl<Ctx> Decodable for Proof<Ctx>
+where
+    Ctx: Swap,
+{
+    fn consensus_decode<D: io::Read>(d: &mut D) -> Result<Self, consensus::Error> {
+        Ok(Self {
+            proof: Ctx::Proof::from_canonical_bytes(unwrap_vec_ref!(d).as_ref())?,
+        })
+    }
+}
+
+impl_strict_encoding!(Proof<Ctx>, Ctx: Swap);
+
 /// Alice parameters required for the initialization step of a swap and used to generate the
 /// [`CommitAliceParameters`] and [`RevealAliceParameters`] protocol messages in the commit/reveal
 /// round.
@@ -31,13 +59,12 @@ pub struct AliceParameters<Ctx: Swap> {
     pub adaptor: <Ctx::Ar as Keys>::PublicKey,
     pub extra_arbitrating_keys: Vec<TaggedElement<u16, <Ctx::Ar as Keys>::PublicKey>>,
     pub arbitrating_shared_keys:
-        Vec<TaggedElement<SharedKeyId, <Ctx::Ar as SharedPrivateKeys>::SharedPrivateKey>>,
+        Vec<TaggedElement<SharedKeyId, <Ctx::Ar as SharedSecretKeys>::SharedSecretKey>>,
     pub spend: <Ctx::Ac as Keys>::PublicKey,
     pub extra_accordant_keys: Vec<TaggedElement<u16, <Ctx::Ac as Keys>::PublicKey>>,
     pub accordant_shared_keys:
-        Vec<TaggedElement<SharedKeyId, <Ctx::Ac as SharedPrivateKeys>::SharedPrivateKey>>,
+        Vec<TaggedElement<SharedKeyId, <Ctx::Ac as SharedSecretKeys>::SharedSecretKey>>,
     pub destination_address: <Ctx::Ar as Address>::Address,
-    pub proof: Ctx::Proof,
     pub cancel_timelock: Option<<Ctx::Ar as Timelock>::Timelock>,
     pub punish_timelock: Option<<Ctx::Ar as Timelock>::Timelock>,
     pub fee_strategy: Option<FeeStrategy<<Ctx::Ar as Fee>::FeeUnit>>,
@@ -62,7 +89,6 @@ where
             .destination_address
             .as_canonical_bytes()
             .consensus_encode(s)?;
-        len += self.proof.as_canonical_bytes().consensus_encode(s)?;
         len += self.cancel_timelock.consensus_encode(s)?;
         len += self.punish_timelock.consensus_encode(s)?;
         Ok(len + self.fee_strategy.consensus_encode(s)?)
@@ -96,7 +122,6 @@ where
             destination_address: <Ctx::Ar as Address>::Address::from_canonical_bytes(
                 unwrap_vec_ref!(d).as_ref(),
             )?,
-            proof: Ctx::Proof::from_canonical_bytes(unwrap_vec_ref!(d).as_ref())?,
             cancel_timelock: Decodable::consensus_decode(d)?,
             punish_timelock: Decodable::consensus_decode(d)?,
             fee_strategy: Decodable::consensus_decode(d)?,
@@ -123,7 +148,6 @@ where
             extra_accordant_keys: msg.extra_accordant_keys,
             accordant_shared_keys: msg.accordant_shared_keys,
             destination_address: msg.address,
-            proof: msg.proof,
             cancel_timelock: None,
             punish_timelock: None,
             fee_strategy: None,
@@ -146,13 +170,12 @@ pub struct BobParameters<Ctx: Swap> {
     pub adaptor: <Ctx::Ar as Keys>::PublicKey,
     pub extra_arbitrating_keys: Vec<TaggedElement<u16, <Ctx::Ar as Keys>::PublicKey>>,
     pub arbitrating_shared_keys:
-        Vec<TaggedElement<SharedKeyId, <Ctx::Ar as SharedPrivateKeys>::SharedPrivateKey>>,
+        Vec<TaggedElement<SharedKeyId, <Ctx::Ar as SharedSecretKeys>::SharedSecretKey>>,
     pub spend: <Ctx::Ac as Keys>::PublicKey,
     pub extra_accordant_keys: Vec<TaggedElement<u16, <Ctx::Ac as Keys>::PublicKey>>,
     pub accordant_shared_keys:
-        Vec<TaggedElement<SharedKeyId, <Ctx::Ac as SharedPrivateKeys>::SharedPrivateKey>>,
+        Vec<TaggedElement<SharedKeyId, <Ctx::Ac as SharedSecretKeys>::SharedSecretKey>>,
     pub refund_address: <Ctx::Ar as Address>::Address,
-    pub proof: Ctx::Proof,
     pub cancel_timelock: Option<<Ctx::Ar as Timelock>::Timelock>,
     pub punish_timelock: Option<<Ctx::Ar as Timelock>::Timelock>,
     pub fee_strategy: Option<FeeStrategy<<Ctx::Ar as Fee>::FeeUnit>>,
@@ -176,7 +199,6 @@ where
             .refund_address
             .as_canonical_bytes()
             .consensus_encode(s)?;
-        len += self.proof.as_canonical_bytes().consensus_encode(s)?;
         len += self.cancel_timelock.consensus_encode(s)?;
         len += self.punish_timelock.consensus_encode(s)?;
         Ok(len + self.fee_strategy.consensus_encode(s)?)
@@ -207,7 +229,6 @@ where
             refund_address: <Ctx::Ar as Address>::Address::from_canonical_bytes(
                 unwrap_vec_ref!(d).as_ref(),
             )?,
-            proof: Ctx::Proof::from_canonical_bytes(unwrap_vec_ref!(d).as_ref())?,
             cancel_timelock: Decodable::consensus_decode(d)?,
             punish_timelock: Decodable::consensus_decode(d)?,
             fee_strategy: Decodable::consensus_decode(d)?,
@@ -233,7 +254,6 @@ where
             extra_accordant_keys: msg.extra_accordant_keys,
             accordant_shared_keys: msg.accordant_shared_keys,
             refund_address: msg.address,
-            proof: msg.proof,
             cancel_timelock: None,
             punish_timelock: None,
             fee_strategy: None,
@@ -420,7 +440,7 @@ where
     T: Signatures + Onchain,
 {
     pub buy: T::PartialTransaction,
-    pub buy_adaptor_sig: T::AdaptorSignature,
+    pub buy_adaptor_sig: T::EncryptedSignature,
 }
 
 fn signed_adaptor_buy_fmt<T>(b: &SignedAdaptorBuy<T>) -> String
@@ -454,7 +474,7 @@ where
     fn consensus_decode<D: io::Read>(d: &mut D) -> Result<Self, consensus::Error> {
         Ok(Self {
             buy: T::PartialTransaction::from_canonical_bytes(unwrap_vec_ref!(d).as_ref())?,
-            buy_adaptor_sig: T::AdaptorSignature::from_canonical_bytes(
+            buy_adaptor_sig: T::EncryptedSignature::from_canonical_bytes(
                 unwrap_vec_ref!(d).as_ref(),
             )?,
         })
@@ -517,7 +537,7 @@ pub struct SignedAdaptorRefund<S>
 where
     S: Signatures,
 {
-    pub refund_adaptor_sig: S::AdaptorSignature,
+    pub refund_adaptor_sig: S::EncryptedSignature,
 }
 
 impl<S> Encodable for SignedAdaptorRefund<S>
@@ -537,7 +557,7 @@ where
 {
     fn consensus_decode<D: io::Read>(d: &mut D) -> Result<Self, consensus::Error> {
         Ok(Self {
-            refund_adaptor_sig: S::AdaptorSignature::from_canonical_bytes(
+            refund_adaptor_sig: S::EncryptedSignature::from_canonical_bytes(
                 unwrap_vec_ref!(d).as_ref(),
             )?,
         })
