@@ -2,13 +2,23 @@
 //! sign, etc) used to create the generic framework for supporting multiple blockchains under the
 //! same interface.
 
+use std::convert::TryInto;
 use std::error;
 use std::fmt::{self, Debug};
 use std::io;
 
 use thiserror::Error;
+use tiny_keccak::{Hasher, Keccak};
 
 use crate::consensus::{self, CanonicalBytes, Decodable, Encodable};
+use crate::role::Accordant;
+
+#[cfg(feature = "experimental")]
+#[cfg_attr(docsrs, doc(cfg(feature = "experimental")))]
+pub mod dleq;
+#[cfg(feature = "experimental")]
+#[cfg_attr(docsrs, doc(cfg(feature = "experimental")))]
+pub mod slip10;
 
 /// List of cryptographic errors that can be encountered in cryptographic operations such as
 /// signatures, proofs, key derivation, or commitments.
@@ -17,18 +27,36 @@ pub enum Error {
     /// The key identifier is not supported and the key cannot be derived.
     #[error("The key identifier is not supported and the key cannot be derived")]
     UnsupportedKey,
+    /// The key or key identifier does not exists or is missing.
+    #[error("The key or key identifier does not exists or is missing")]
+    MissingKey,
     /// The signature does not pass the validation tests.
     #[error("The signature does not pass the validation")]
     InvalidSignature,
+    /// The adaptor key is not valid.
+    #[error("The adaptor key is not valid")]
+    InvalidAdaptorKey,
     /// The adaptor signature does not pass the validation tests.
     #[error("The adaptor signature does not pass the validation")]
-    InvalidAdaptorSignature,
+    InvalidEncryptedSignature,
     /// The proof does not pass the validation tests.
     #[error("The proof does not pass the validation")]
     InvalidProof,
     /// The commitment does not match the given value.
     #[error("The commitment does not match the given value")]
     InvalidCommitment,
+    /// The Pedersen commitment does not match the given value.
+    #[error("The Pedersen commitment does not match the given value")]
+    InvalidPedersenCommitment,
+    /// The ring signature does not recompute.
+    #[error("The ring signature does not recompute")]
+    InvalidRingSignature,
+    /// The proof of knowledge signature is invalid.
+    #[error("The proof of knowledge signature is invalid")]
+    InvalidProofOfKnowledge,
+    /// SLIP10 error when manipulating extended secret keys.
+    #[error("SLIP10 error: {0}")]
+    Slip10(#[from] slip10::Error),
     /// Any cryptographic error not part of this list.
     #[error("Cryptographic error: {0}")]
     Other(Box<dyn error::Error + Send + Sync>),
@@ -150,10 +178,11 @@ pub type TaggedSharedKeys<E> = Vec<TaggedElement<SharedKeyId, E>>;
     serde(crate = "serde_crate")
 )]
 pub enum ArbitratingKeyId {
-    /// Arbitrating key used to fund the swap in [`Fundable`].
+    /// Arbitrating key used to fund the [`Lockable`] transaction through [`Fundable`].
     ///
+    /// [`Lockable`]: crate::transaction::Lockable
     /// [`Fundable`]: crate::transaction::Fundable
-    Fund,
+    Lock,
     /// Key used in the [`Buyable`] transaction.
     ///
     /// [`Buyable`]: crate::transaction::Buyable
@@ -230,6 +259,73 @@ impl Decodable for SharedKeyId {
     }
 }
 
+/// The list of possible [`Accordant`] keys (secret and public) a swap role has after reveal.
+pub struct AccordantKeys<A: Accordant> {
+    /// The accordant spend public key.
+    pub spend_key: A::PublicKey,
+    /// A list of extra accordant public keys.
+    pub extra_accordant_keys: Vec<TaggedElement<u16, A::PublicKey>>,
+    /// A list of secret shared keys, e.g. shared view keys in non-transparent blockchains.
+    pub shared_keys: Vec<TaggedElement<SharedKeyId, A::SharedSecretKey>>,
+}
+
+/// The list of all accordant keys swap roles have after reveal.
+pub struct SwapAccordantKeys<A: Accordant> {
+    /// Alice's accordant keys (secret and public).
+    pub alice: AccordantKeys<A>,
+    /// Bob's accordant keys (secret and public).
+    pub bob: AccordantKeys<A>,
+}
+
+fixed_hash::construct_fixed_hash!(
+    /// Result of a keccak256 commitment.
+    #[cfg_attr(
+        feature = "serde",
+        derive(Serialize, Deserialize),
+        serde(crate = "serde_crate"),
+    )]
+    pub struct KeccakCommitment(32);
+);
+
+impl KeccakCommitment {
+    /// Create a null commitment hash with all zeros.
+    pub fn null_hash() -> Self {
+        Self([0u8; 32])
+    }
+
+    /// Hash a stream of bytes with the Keccak-256 hash function.
+    pub fn new(input: [u8; 32]) -> Self {
+        Self(input)
+    }
+}
+
+impl CanonicalBytes for KeccakCommitment {
+    fn as_canonical_bytes(&self) -> Vec<u8> {
+        (*self).to_fixed_bytes().into()
+    }
+
+    fn from_canonical_bytes(bytes: &[u8]) -> Result<Self, consensus::Error>
+    where
+        Self: Sized,
+    {
+        Ok(Self::new(bytes.try_into().map_err(consensus::Error::new)?))
+    }
+}
+
+/// Engine to produce and validate hash commitments.
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub struct CommitmentEngine;
+
+impl Commit<KeccakCommitment> for CommitmentEngine {
+    fn commit_to<T: AsRef<[u8]>>(&self, value: T) -> KeccakCommitment {
+        let mut out = [0u8; 32];
+        let mut keccak = Keccak::v256();
+        keccak.update(value.as_ref());
+        keccak.finalize(&mut out);
+        KeccakCommitment::new(out)
+    }
+}
+
 /// Required for [`Arbitrating`] and [`Accordant`] blockchains to fix the cryptographic secret key
 /// and public key types. The public key type is shared across the network and used in
 /// transactions, the secret key type is used during signing operation, proofs, etc.
@@ -238,7 +334,7 @@ impl Decodable for SharedKeyId {
 /// [`Accordant`]: crate::role::Accordant
 pub trait Keys {
     /// Secret key type used for signing and proving.
-    type PrivateKey;
+    type SecretKey;
 
     /// Public key type used in transactions.
     type PublicKey: Clone + PartialEq + Debug + fmt::Display + CanonicalBytes;
@@ -252,21 +348,14 @@ pub trait Keys {
 ///
 /// [`Arbitrating`]: crate::role::Arbitrating
 /// [`Accordant`]: crate::role::Accordant
-pub trait SharedPrivateKeys {
+pub trait SharedSecretKeys {
     /// Shareable secret key type used to parse, e.g., non-transparent blockchain.
-    type SharedPrivateKey: Clone + PartialEq + Debug + CanonicalBytes;
+    type SharedSecretKey: Clone + PartialEq + Debug + CanonicalBytes;
 
     /// Return a list of extra shared secret key identifiers to use during the setup phase.
     fn shared_keys() -> Vec<SharedKeyId>;
 }
 
-/// Required for swaps to fix the commitments type of the keys and parameters that must go through
-/// the commit/reveal scheme at the beginning of the protocol.
-pub trait Commitment {
-    /// Commitment type used in the commit/reveal scheme during swap setup.
-    type Commitment: Clone + PartialEq + Eq + Debug + fmt::Display + CanonicalBytes;
-}
-
 /// Trait required for [`Arbitrating`] blockchains to define the cryptographic message format to
 /// sign, the signature format and adaptor signature format used in the cryptographic operations
 /// such as signing and verifying signatures and adaptor signatures.
@@ -282,16 +371,16 @@ pub trait Signatures {
 
     /// Defines the adaptor signature format for the arbitrating blockchain. Adaptor signature may
     /// have a different format from the signature depending on the cryptographic primitives used.
-    type AdaptorSignature: Clone + Debug + fmt::Display + CanonicalBytes;
+    type EncryptedSignature: Clone + Debug + fmt::Display + CanonicalBytes;
 }
 
-/// Meta trait regrouping all the needed trait combinaisons a key manager must implement. Used when
-/// executing the protocol on [`Alice`] and [`Bob`] methods. This trait is auto-implemented for all
-/// `T` meeting the requirements.
+/// Meta trait regrouping all the needed trait combinations a key manager must implement to manage
+/// all the keys needed when executing the protocol on [`Alice`] and [`Bob`] methods. This trait is
+/// auto-implemented for all `T` meeting the requirements.
 ///
 /// [`Alice`]: crate::role::Alice
 /// [`Bob`]: crate::role::Bob
-pub trait Wallet<ArPublicKey, AcPublicKey, ArSharedKey, AcSharedKey, Proof>:
+pub trait KeyGenerator<ArPublicKey, AcPublicKey, ArSharedKey, AcSharedKey, Proof>:
     GenerateKey<ArPublicKey, ArbitratingKeyId>
     + GenerateKey<AcPublicKey, AccordantKeyId>
     + ProveCrossGroupDleq<ArPublicKey, AcPublicKey, Proof>
@@ -301,7 +390,7 @@ pub trait Wallet<ArPublicKey, AcPublicKey, ArSharedKey, AcSharedKey, Proof>:
 }
 
 impl<T, ArPublicKey, AcPublicKey, ArSharedKey, AcSharedKey, Proof>
-    Wallet<ArPublicKey, AcPublicKey, ArSharedKey, AcSharedKey, Proof> for T
+    KeyGenerator<ArPublicKey, AcPublicKey, ArSharedKey, AcSharedKey, Proof> for T
 where
     T: GenerateKey<ArPublicKey, ArbitratingKeyId>
         + GenerateKey<AcPublicKey, AccordantKeyId>
@@ -317,11 +406,11 @@ pub trait GenerateKey<PublicKey, KeyId> {
     /// Retreive a specific public key by its key id. If the key cannot be derived the
     /// implementation must return an [`Error::UnsupportedKey`], otherwise `Ok(PublicKey)` is
     /// returned.
-    fn get_pubkey(&self, key_id: KeyId) -> Result<PublicKey, Error>;
+    fn get_pubkey(&mut self, key_id: KeyId) -> Result<PublicKey, Error>;
 
     /// Return a vector of public keys matching the vector of key ids. Errors on the first key that
     /// can't be derived and return an [`Error::UnsupportedKey`].
-    fn get_pubkeys(&self, key_ids: Vec<KeyId>) -> Result<Vec<PublicKey>, Error> {
+    fn get_pubkeys(&mut self, key_ids: Vec<KeyId>) -> Result<Vec<PublicKey>, Error> {
         key_ids.into_iter().map(|id| self.get_pubkey(id)).collect()
     }
 }
@@ -331,52 +420,59 @@ pub trait GenerateKey<PublicKey, KeyId> {
 pub trait GenerateSharedKey<SharedKey> {
     /// Retreive a specific shared private key by its key id. If the key cannot be derived the
     /// implementation must return an [`Error::UnsupportedKey`].
-    fn get_shared_key(&self, key_id: SharedKeyId) -> Result<SharedKey, Error>;
+    fn get_shared_key(&mut self, key_id: SharedKeyId) -> Result<SharedKey, Error>;
 }
 
 // TODO give extra keys and/or shared keys in signing methods
 
-/// Signature and adaptor signature generator and verifier. Produce and verify signatures and
-/// adaptor sigantures based on public keys. Recover the private key through the complete
-/// adaptor/adapted signature.
-pub trait Sign<PublicKey, PrivateKey, Message, Signature, AdaptorSignature> {
-    /// Sign the message with the corresponding private key identified by the provided public key.
-    fn sign_with_key(&self, key: &PublicKey, msg: Message) -> Result<Signature, Error>;
+/// Signature and encrypted signature generator and verifier. Produce and verify signatures and
+/// encrypted sigantures based on public keys/key identifiers. Recover the private key through the
+/// complete encrypted/decrypted signature.
+pub trait Sign<PublicKey, SecretKey, Message, Signature, EncryptedSignature> {
+    /// Sign the message with the corresponding private key identified by the provided arbitrating
+    /// key identifier.
+    fn sign(&mut self, key: ArbitratingKeyId, msg: Message) -> Result<Signature, Error>;
 
     /// Verify a signature for a given message with the provided public key.
     fn verify_signature(&self, key: &PublicKey, msg: Message, sig: &Signature)
         -> Result<(), Error>;
 
-    /// Sign the message with the corresponding private key identified by the provided public key
-    /// and encrypt it (create an adaptor signature) with the provided adaptor public key.
-    fn adaptor_sign_with_key(
-        &self,
-        key: &PublicKey,
-        adaptor: &PublicKey,
+    /// Sign the message with the corresponding private key identified by the provided arbitrating
+    /// key identifier and encrypt it (create an adaptor signature) with the provided encryption
+    /// public key.
+    fn encrypt_sign(
+        &mut self,
+        signing_key: ArbitratingKeyId,
+        encryption_key: &PublicKey,
         msg: Message,
-    ) -> Result<AdaptorSignature, Error>;
+    ) -> Result<EncryptedSignature, Error>;
 
-    /// Verify a adaptor signature for a given message with the provided public key and the public
-    /// adaptor key.
-    fn verify_adaptor_signature(
+    /// Verify an encrypted signature for a given message with the provided signing public key and
+    /// the public encryption key.
+    fn verify_encrypted_signature(
         &self,
-        key: &PublicKey,
-        adaptor: &PublicKey,
+        signing_key: &PublicKey,
+        encryption_key: &PublicKey,
         msg: Message,
-        sig: &AdaptorSignature,
+        sig: &EncryptedSignature,
     ) -> Result<(), Error>;
 
-    /// Finalize an adaptor signature (decrypt the signature) into an adapted signature (decrypted
-    /// signatures) with the corresponding private key identified by the provided public key.
-    fn adapt_signature(&self, key: &PublicKey, sig: AdaptorSignature) -> Result<Signature, Error>;
+    /// Decrypt the encrypted signature with the corresponding decryption private key identified by
+    /// the provided accordant key identifier, producing a valid regular signature.
+    fn decrypt_signature(
+        &mut self,
+        decryption_key: AccordantKeyId,
+        sig: EncryptedSignature,
+    ) -> Result<Signature, Error>;
 
-    /// Recover the encryption key based on the adaptor signature and the decrypted signature.
-    fn recover_key(
+    /// Recover the encryption key based on the encrypted signature, the encryption public key, and
+    /// the regular (decrypted) signature.
+    fn recover_secret_key(
         &self,
-        adaptor_key: &PublicKey,
+        encrypted_sig: EncryptedSignature,
+        encryption_key: &PublicKey,
         sig: Signature,
-        adapted_sig: AdaptorSignature,
-    ) -> PrivateKey;
+    ) -> SecretKey;
 }
 
 /// Commitment generator and verifier. Generated commitments can be validated against candidates,
@@ -397,22 +493,23 @@ pub trait Commit<Commitment: Eq> {
     }
 }
 
-/// Proof generator and verifier for the cross-group projection of the accordant public spend key.
-pub trait ProveCrossGroupDleq<Adaptor, PublicSpendKey, Proof> {
-    /// Generate the proof and the two public keys: the arbitrating public key, also called the
-    /// adaptor public key, and the accordant public spend key.
-    fn generate(&self) -> Result<(PublicSpendKey, Adaptor, Proof), Error>;
+/// Proof generator and verifier for the cross-group projection of the accordant public spend key
+/// as an arbitrating key used to encrypt signatures.
+pub trait ProveCrossGroupDleq<EncryptionKey, AccordantSpendKey, Proof> {
+    /// Generate the proof and the two public keys: the accordant public spend key and the
+    /// arbitrating public key, also called the encryption public key,
+    fn generate_proof(&mut self) -> Result<(AccordantSpendKey, EncryptionKey, Proof), Error>;
 
-    /// Project the accordant sepnd secret key over the arbitrating curve to get the public key
-    /// used as the adaptor public key.
-    fn project_over(&self) -> Result<Adaptor, Error>;
+    /// Project the accordant spend secret key over the arbitrating curve to get the public key
+    /// used as the encryption public key.
+    fn get_encryption_key(&mut self) -> Result<EncryptionKey, Error>;
 
     /// Verify the proof given the two public keys: the accordant spend public key and the
-    /// arbitrating adaptor public key.
-    fn verify(
-        &self,
-        public_spend: &PublicSpendKey,
-        adaptor: &Adaptor,
+    /// arbitrating encryption public key.
+    fn verify_proof(
+        &mut self,
+        public_spend: &AccordantSpendKey,
+        encryption_key: &EncryptionKey,
         proof: Proof,
     ) -> Result<(), Error>;
 }
