@@ -6,16 +6,18 @@ use std::fmt::Debug;
 use std::io;
 use std::str::FromStr;
 
-use crate::blockchain::{Address, Asset, Fee, FeePriority, Onchain, Timelock, Transactions};
+use crate::blockchain::{
+    Address, Asset, Fee, FeePriority, Network, Onchain, Timelock, Transactions,
+};
 use crate::bundle::{
     AliceParameters, BobParameters, CoreArbitratingTransactions, CosignedArbitratingCancel,
-    FullySignedBuy, FullySignedPunish, FullySignedRefund, SignedAdaptorBuy, SignedAdaptorRefund,
-    SignedArbitratingLock,
+    FullySignedBuy, FullySignedPunish, FullySignedRefund, Proof, SignedAdaptorBuy,
+    SignedAdaptorRefund, SignedArbitratingLock,
 };
 use crate::consensus::{self, Decodable, Encodable};
 use crate::crypto::{
-    AccordantKeyId, ArbitratingKeyId, Keys, SharedPrivateKeys, Sign, Signatures, TaggedElement,
-    TaggedExtraKeys, TaggedSharedKeys, Wallet,
+    self, AccordantKeyId, ArbitratingKeyId, KeyGenerator, Keys, SharedSecretKeys, Sign, Signatures,
+    SwapAccordantKeys, TaggedElement, TaggedExtraKeys, TaggedSharedKeys,
 };
 use crate::negotiation::PublicOffer;
 use crate::script::{DataLock, DataPunishableLock, DoubleKeys, ScriptPath};
@@ -174,12 +176,11 @@ pub struct Alice<Ctx: Swap> {
     pub fee_politic: FeePriority,
 }
 
-struct ValidatedCoreTransactions<Ctx: Swap> {
+struct ValidatedCoreTransactions<'a, Ctx: Swap> {
     lock: <Ctx::Ar as Transactions>::Lock,
     cancel: <Ctx::Ar as Transactions>::Cancel,
     refund: <Ctx::Ar as Transactions>::Refund,
-    data_lock: DataLock<Ctx::Ar>,
-    punish_lock: DataPunishableLock<Ctx::Ar>,
+    punish_lock: DataPunishableLock<'a, Ctx::Ar>,
 }
 
 impl<Ctx> Alice<Ctx>
@@ -214,30 +215,30 @@ where
     ///
     pub fn generate_parameters(
         &self,
-        wallet: &impl Wallet<
+        key_gen: &mut impl KeyGenerator<
             <Ctx::Ar as Keys>::PublicKey,
             <Ctx::Ac as Keys>::PublicKey,
-            <Ctx::Ar as SharedPrivateKeys>::SharedPrivateKey,
-            <Ctx::Ac as SharedPrivateKeys>::SharedPrivateKey,
+            <Ctx::Ar as SharedSecretKeys>::SharedSecretKey,
+            <Ctx::Ac as SharedSecretKeys>::SharedSecretKey,
             Ctx::Proof,
         >,
         public_offer: &PublicOffer<Ctx>,
-    ) -> Res<AliceParameters<Ctx>> {
+    ) -> Res<(AliceParameters<Ctx>, Proof<Ctx>)> {
         let extra_arbitrating_keys: Res<TaggedExtraKeys<<Ctx::Ar as Keys>::PublicKey>> =
             <Ctx::Ar as Keys>::extra_keys()
                 .into_iter()
                 .map(|tag| {
-                    let key = wallet.get_pubkey(ArbitratingKeyId::Extra(tag))?;
+                    let key = key_gen.get_pubkey(ArbitratingKeyId::Extra(tag))?;
                     Ok(TaggedElement::new(tag, key))
                 })
                 .collect();
 
         let arbitrating_shared_keys: Res<
-            TaggedSharedKeys<<Ctx::Ar as SharedPrivateKeys>::SharedPrivateKey>,
-        > = <Ctx::Ar as SharedPrivateKeys>::shared_keys()
+            TaggedSharedKeys<<Ctx::Ar as SharedSecretKeys>::SharedSecretKey>,
+        > = <Ctx::Ar as SharedSecretKeys>::shared_keys()
             .into_iter()
             .map(|tag| {
-                let key = wallet.get_shared_key(tag)?;
+                let key = key_gen.get_shared_key(tag)?;
                 Ok(TaggedElement::new(tag, key))
             })
             .collect();
@@ -246,40 +247,42 @@ where
             <Ctx::Ac as Keys>::extra_keys()
                 .into_iter()
                 .map(|tag| {
-                    let key = wallet.get_pubkey(AccordantKeyId::Extra(tag))?;
+                    let key = key_gen.get_pubkey(AccordantKeyId::Extra(tag))?;
                     Ok(TaggedElement::new(tag, key))
                 })
                 .collect();
 
         let accordant_shared_keys: Res<
-            TaggedSharedKeys<<Ctx::Ac as SharedPrivateKeys>::SharedPrivateKey>,
-        > = <Ctx::Ac as SharedPrivateKeys>::shared_keys()
+            TaggedSharedKeys<<Ctx::Ac as SharedSecretKeys>::SharedSecretKey>,
+        > = <Ctx::Ac as SharedSecretKeys>::shared_keys()
             .into_iter()
             .map(|tag| {
-                let key = wallet.get_shared_key(tag)?;
+                let key = key_gen.get_shared_key(tag)?;
                 Ok(TaggedElement::new(tag, key))
             })
             .collect();
 
-        let (spend, adaptor, proof) = wallet.generate()?;
+        let (spend, adaptor, proof) = key_gen.generate_proof()?;
 
-        Ok(AliceParameters {
-            buy: wallet.get_pubkey(ArbitratingKeyId::Buy)?,
-            cancel: wallet.get_pubkey(ArbitratingKeyId::Cancel)?,
-            refund: wallet.get_pubkey(ArbitratingKeyId::Refund)?,
-            punish: wallet.get_pubkey(ArbitratingKeyId::Punish)?,
-            adaptor,
-            extra_arbitrating_keys: extra_arbitrating_keys?,
-            arbitrating_shared_keys: arbitrating_shared_keys?,
-            spend,
-            extra_accordant_keys: extra_accordant_keys?,
-            accordant_shared_keys: accordant_shared_keys?,
-            destination_address: self.destination_address.clone(),
-            proof,
-            cancel_timelock: Some(public_offer.offer.cancel_timelock),
-            punish_timelock: Some(public_offer.offer.punish_timelock),
-            fee_strategy: Some(public_offer.offer.fee_strategy.clone()),
-        })
+        Ok((
+            AliceParameters {
+                buy: key_gen.get_pubkey(ArbitratingKeyId::Buy)?,
+                cancel: key_gen.get_pubkey(ArbitratingKeyId::Cancel)?,
+                refund: key_gen.get_pubkey(ArbitratingKeyId::Refund)?,
+                punish: key_gen.get_pubkey(ArbitratingKeyId::Punish)?,
+                adaptor,
+                extra_arbitrating_keys: extra_arbitrating_keys?,
+                arbitrating_shared_keys: arbitrating_shared_keys?,
+                spend,
+                extra_accordant_keys: extra_accordant_keys?,
+                accordant_shared_keys: accordant_shared_keys?,
+                destination_address: self.destination_address.clone(),
+                cancel_timelock: Some(public_offer.offer.cancel_timelock),
+                punish_timelock: Some(public_offer.offer.punish_timelock),
+                fee_strategy: Some(public_offer.offer.fee_strategy.clone()),
+            },
+            Proof { proof },
+        ))
     }
 
     /// Generates the witness on the [`Refundable`] transaction and adaptor sign it.
@@ -318,12 +321,12 @@ where
     ///
     pub fn sign_adaptor_refund(
         &self,
-        wallet: &impl Sign<
+        wallet: &mut impl Sign<
             <Ctx::Ar as Keys>::PublicKey,
-            <Ctx::Ar as Keys>::PrivateKey,
+            <Ctx::Ar as Keys>::SecretKey,
             <Ctx::Ar as Signatures>::Message,
             <Ctx::Ar as Signatures>::Signature,
-            <Ctx::Ar as Signatures>::AdaptorSignature,
+            <Ctx::Ar as Signatures>::EncryptedSignature,
         >,
         alice_parameters: &AliceParameters<Ctx>,
         bob_parameters: &BobParameters<Ctx>,
@@ -336,10 +339,9 @@ where
 
         // Generate the witness message to sign and adaptor sign with the refund key and the
         // counter-party adaptor.
-        let key = &alice_parameters.refund;
         let adaptor = &bob_parameters.adaptor;
         let msg = refund.generate_witness_message(ScriptPath::Success)?;
-        let sig = wallet.adaptor_sign_with_key(key, adaptor, msg)?;
+        let sig = wallet.encrypt_sign(ArbitratingKeyId::Refund, adaptor, msg)?;
 
         Ok(SignedAdaptorRefund {
             refund_adaptor_sig: sig,
@@ -375,12 +377,12 @@ where
     ///
     pub fn cosign_arbitrating_cancel(
         &self,
-        wallet: &impl Sign<
+        wallet: &mut impl Sign<
             <Ctx::Ar as Keys>::PublicKey,
-            <Ctx::Ar as Keys>::PrivateKey,
+            <Ctx::Ar as Keys>::SecretKey,
             <Ctx::Ar as Signatures>::Message,
             <Ctx::Ar as Signatures>::Signature,
-            <Ctx::Ar as Signatures>::AdaptorSignature,
+            <Ctx::Ar as Signatures>::EncryptedSignature,
         >,
         alice_parameters: &AliceParameters<Ctx>,
         bob_parameters: &BobParameters<Ctx>,
@@ -393,8 +395,7 @@ where
 
         // Generate the witness message to sign and sign with the cancel key.
         let msg = cancel.generate_witness_message(ScriptPath::Failure)?;
-        let key = &alice_parameters.cancel;
-        let sig = wallet.sign_with_key(key, msg)?;
+        let sig = wallet.sign(ArbitratingKeyId::Cancel, msg)?;
 
         Ok(CosignedArbitratingCancel { cancel_sig: sig })
     }
@@ -429,12 +430,12 @@ where
     ///
     pub fn validate_adaptor_buy(
         &self,
-        wallet: &impl Sign<
+        wallet: &mut impl Sign<
             <Ctx::Ar as Keys>::PublicKey,
-            <Ctx::Ar as Keys>::PrivateKey,
+            <Ctx::Ar as Keys>::SecretKey,
             <Ctx::Ar as Signatures>::Message,
             <Ctx::Ar as Signatures>::Signature,
-            <Ctx::Ar as Signatures>::AdaptorSignature,
+            <Ctx::Ar as Signatures>::EncryptedSignature,
         >,
         alice_parameters: &AliceParameters<Ctx>,
         bob_parameters: &BobParameters<Ctx>,
@@ -443,9 +444,8 @@ where
         adaptor_buy: &SignedAdaptorBuy<Ctx::Ar>,
     ) -> Res<()> {
         // Verifies the core arbitrating transactions.
-        let ValidatedCoreTransactions {
-            lock, data_lock, ..
-        } = self.validate_core(alice_parameters, bob_parameters, core, public_offer)?;
+        let ValidatedCoreTransactions { lock, .. } =
+            self.validate_core(alice_parameters, bob_parameters, core, public_offer)?;
 
         let fee_strategy = &public_offer.offer.fee_strategy;
 
@@ -457,12 +457,12 @@ where
         let buy = <<Ctx::Ar as Transactions>::Buy>::from_partial(partial_buy);
 
         buy.is_build_on_top_of(&lock)?;
-        buy.verify_template(data_lock, self.destination_address.clone())?;
+        buy.verify_template(self.destination_address.clone())?;
         <Ctx::Ar as Fee>::validate_fee(buy.as_partial(), fee_strategy)?;
 
         // Verify the adaptor buy witness
         let msg = buy.generate_witness_message(ScriptPath::Success)?;
-        wallet.verify_adaptor_signature(
+        wallet.verify_encrypted_signature(
             &bob_parameters.buy,
             &alice_parameters.adaptor,
             msg,
@@ -510,12 +510,12 @@ where
     ///
     pub fn fully_sign_buy(
         &self,
-        wallet: &impl Sign<
+        wallet: &mut impl Sign<
             <Ctx::Ar as Keys>::PublicKey,
-            <Ctx::Ar as Keys>::PrivateKey,
+            <Ctx::Ar as Keys>::SecretKey,
             <Ctx::Ar as Signatures>::Message,
             <Ctx::Ar as Signatures>::Signature,
-            <Ctx::Ar as Signatures>::AdaptorSignature,
+            <Ctx::Ar as Signatures>::EncryptedSignature,
         >,
         alice_parameters: &AliceParameters<Ctx>,
         bob_parameters: &BobParameters<Ctx>,
@@ -524,9 +524,8 @@ where
         adaptor_buy: &SignedAdaptorBuy<Ctx::Ar>,
     ) -> Res<FullySignedBuy<Ctx::Ar>> {
         // Verifies the core arbitrating transactions.
-        let ValidatedCoreTransactions {
-            lock, data_lock, ..
-        } = self.validate_core(alice_parameters, bob_parameters, core, public_offer)?;
+        let ValidatedCoreTransactions { lock, .. } =
+            self.validate_core(alice_parameters, bob_parameters, core, public_offer)?;
 
         let fee_strategy = &public_offer.offer.fee_strategy;
 
@@ -538,17 +537,16 @@ where
         let buy = <<Ctx::Ar as Transactions>::Buy>::from_partial(partial_buy);
 
         buy.is_build_on_top_of(&lock)?;
-        buy.verify_template(data_lock, self.destination_address.clone())?;
+        buy.verify_template(self.destination_address.clone())?;
         <Ctx::Ar as Fee>::validate_fee(buy.as_partial(), fee_strategy)?;
 
         // Generate the witness message to sign and sign with the buy key.
         let msg = buy.generate_witness_message(ScriptPath::Success)?;
-        let key = &alice_parameters.buy;
-        let sig = wallet.sign_with_key(key, msg)?;
+        let sig = wallet.sign(ArbitratingKeyId::Buy, msg)?;
 
         // Retreive the adaptor public key and the counter-party adaptor witness.
-        let key = &alice_parameters.adaptor;
-        let adapted_sig = wallet.adapt_signature(key, adaptor_buy.buy_adaptor_sig.clone())?;
+        let adapted_sig =
+            wallet.decrypt_signature(AccordantKeyId::Spend, adaptor_buy.buy_adaptor_sig.clone())?;
 
         Ok(FullySignedBuy {
             buy_sig: sig,
@@ -590,12 +588,12 @@ where
     ///
     pub fn fully_sign_punish(
         &self,
-        wallet: &impl Sign<
+        wallet: &mut impl Sign<
             <Ctx::Ar as Keys>::PublicKey,
-            <Ctx::Ar as Keys>::PrivateKey,
+            <Ctx::Ar as Keys>::SecretKey,
             <Ctx::Ar as Signatures>::Message,
             <Ctx::Ar as Signatures>::Signature,
-            <Ctx::Ar as Signatures>::AdaptorSignature,
+            <Ctx::Ar as Signatures>::EncryptedSignature,
         >,
         alice_parameters: &AliceParameters<Ctx>,
         bob_parameters: &BobParameters<Ctx>,
@@ -623,8 +621,7 @@ where
 
         // Generate the witness message to sign and sign with the punish key.
         let msg = punish.generate_witness_message(ScriptPath::Failure)?;
-        let key = &alice_parameters.punish;
-        let punish_sig = wallet.sign_with_key(key, msg)?;
+        let punish_sig = wallet.sign(ArbitratingKeyId::Punish, msg)?;
 
         Ok(FullySignedPunish {
             punish: punish.to_partial(),
@@ -632,8 +629,23 @@ where
         })
     }
 
-    pub fn recover_accordant_assets(&self) -> Res<()> {
-        todo!()
+    // TODO: transform into other private key type
+    pub fn recover_accordant_assets(
+        &self,
+        wallet: &mut impl Sign<
+            <Ctx::Ar as Keys>::PublicKey,
+            <Ctx::Ar as Keys>::SecretKey,
+            <Ctx::Ar as Signatures>::Message,
+            <Ctx::Ar as Signatures>::Signature,
+            <Ctx::Ar as Signatures>::EncryptedSignature,
+        >,
+        bob_parameters: &BobParameters<Ctx>,
+        adaptor_refund: SignedAdaptorRefund<Ctx::Ar>,
+        refund_tx: <Ctx::Ar as Onchain>::Transaction,
+    ) -> <Ctx::Ar as Keys>::SecretKey {
+        let encryption_key = &bob_parameters.adaptor;
+        let signature = <<Ctx::Ar as Transactions>::Refund>::extract_witness(refund_tx);
+        wallet.recover_secret_key(adaptor_refund.refund_adaptor_sig, encryption_key, signature)
     }
 
     // Internal method to parse and validate the core arbitratring transactions received by Alice
@@ -647,13 +659,13 @@ where
     //  * the target amount from the offer is correct (for the lock transaction)
     //  * the fee strategy validation passes
     //
-    fn validate_core(
+    fn validate_core<'a>(
         &self,
-        alice_parameters: &AliceParameters<Ctx>,
-        bob_parameters: &BobParameters<Ctx>,
+        alice_parameters: &'a AliceParameters<Ctx>,
+        bob_parameters: &'a BobParameters<Ctx>,
         core: &CoreArbitratingTransactions<Ctx::Ar>,
         public_offer: &PublicOffer<Ctx>,
-    ) -> Res<ValidatedCoreTransactions<Ctx>> {
+    ) -> Res<ValidatedCoreTransactions<'a, Ctx>> {
         // Extract the partial transaction from the core arbitrating bundle, this operation should
         // not error if the bundle is well formed.
         let partial_lock = core.lock.clone();
@@ -664,11 +676,10 @@ where
         // Get the four keys, Alice and Bob for Buy and Cancel. The keys are needed, along with the
         // timelock for the cancel, to create the cancelable on-chain contract on the arbitrating
         // blockchain.
-        // FIXME change dataLock to take refs
-        let alice_buy = alice_parameters.buy.clone();
-        let bob_buy = bob_parameters.buy.clone();
-        let alice_cancel = alice_parameters.cancel.clone();
-        let bob_cancel = bob_parameters.cancel.clone();
+        let alice_buy = &alice_parameters.buy;
+        let bob_buy = &bob_parameters.buy;
+        let alice_cancel = &alice_parameters.cancel;
+        let bob_cancel = &bob_parameters.cancel;
 
         // Create the data structure that represents an on-chain cancelable contract for the
         // arbitrating blockchain.
@@ -691,10 +702,9 @@ where
         // Get the three keys, Alice and Bob for refund and Alice's punish key. The keys are
         // needed, along with the timelock for the punish, to create the punishable on-chain
         // contract on the arbitrating blockchain.
-        // FIXME change dataLock to take refs
-        let alice_refund = alice_parameters.refund.clone();
-        let bob_refund = bob_parameters.refund.clone();
-        let alice_punish = alice_parameters.punish.clone();
+        let alice_refund = &alice_parameters.refund;
+        let bob_refund = &bob_parameters.refund;
+        let alice_punish = &alice_parameters.punish;
 
         // Create the data structure that represents an on-chain punishable contract for the
         // arbitrating blockchain.
@@ -725,7 +735,7 @@ where
         // Check that the refund transaction is build on top of the cancel transaction.
         refund.is_build_on_top_of(&cancel)?;
         let refund_address = bob_parameters.refund_address.clone();
-        refund.verify_template(punish_lock.clone(), refund_address)?;
+        refund.verify_template(refund_address)?;
         // Validate the fee strategy
         <Ctx::Ar as Fee>::validate_fee(refund.as_partial(), fee_strategy)?;
 
@@ -733,7 +743,6 @@ where
             lock,
             cancel,
             refund,
-            data_lock,
             punish_lock,
         })
     }
@@ -776,30 +785,30 @@ impl<Ctx: Swap> Bob<Ctx> {
     ///
     pub fn generate_parameters(
         &self,
-        wallet: &impl Wallet<
+        key_gen: &mut impl KeyGenerator<
             <Ctx::Ar as Keys>::PublicKey,
             <Ctx::Ac as Keys>::PublicKey,
-            <Ctx::Ar as SharedPrivateKeys>::SharedPrivateKey,
-            <Ctx::Ac as SharedPrivateKeys>::SharedPrivateKey,
+            <Ctx::Ar as SharedSecretKeys>::SharedSecretKey,
+            <Ctx::Ac as SharedSecretKeys>::SharedSecretKey,
             Ctx::Proof,
         >,
         public_offer: &PublicOffer<Ctx>,
-    ) -> Res<BobParameters<Ctx>> {
+    ) -> Res<(BobParameters<Ctx>, Proof<Ctx>)> {
         let extra_arbitrating_keys: Res<TaggedExtraKeys<<Ctx::Ar as Keys>::PublicKey>> =
             <Ctx::Ar as Keys>::extra_keys()
                 .into_iter()
                 .map(|tag| {
-                    let key = wallet.get_pubkey(ArbitratingKeyId::Extra(tag))?;
+                    let key = key_gen.get_pubkey(ArbitratingKeyId::Extra(tag))?;
                     Ok(TaggedElement::new(tag, key))
                 })
                 .collect();
 
         let arbitrating_shared_keys: Res<
-            TaggedSharedKeys<<Ctx::Ar as SharedPrivateKeys>::SharedPrivateKey>,
-        > = <Ctx::Ar as SharedPrivateKeys>::shared_keys()
+            TaggedSharedKeys<<Ctx::Ar as SharedSecretKeys>::SharedSecretKey>,
+        > = <Ctx::Ar as SharedSecretKeys>::shared_keys()
             .into_iter()
             .map(|tag| {
-                let key = wallet.get_shared_key(tag)?;
+                let key = key_gen.get_shared_key(tag)?;
                 Ok(TaggedElement::new(tag, key))
             })
             .collect();
@@ -808,39 +817,41 @@ impl<Ctx: Swap> Bob<Ctx> {
             <Ctx::Ac as Keys>::extra_keys()
                 .into_iter()
                 .map(|tag| {
-                    let key = wallet.get_pubkey(AccordantKeyId::Extra(tag))?;
+                    let key = key_gen.get_pubkey(AccordantKeyId::Extra(tag))?;
                     Ok(TaggedElement::new(tag, key))
                 })
                 .collect();
 
         let accordant_shared_keys: Res<
-            TaggedSharedKeys<<Ctx::Ac as SharedPrivateKeys>::SharedPrivateKey>,
-        > = <Ctx::Ac as SharedPrivateKeys>::shared_keys()
+            TaggedSharedKeys<<Ctx::Ac as SharedSecretKeys>::SharedSecretKey>,
+        > = <Ctx::Ac as SharedSecretKeys>::shared_keys()
             .into_iter()
             .map(|tag| {
-                let key = wallet.get_shared_key(tag)?;
+                let key = key_gen.get_shared_key(tag)?;
                 Ok(TaggedElement::new(tag, key))
             })
             .collect();
 
-        let (spend, adaptor, proof) = wallet.generate()?;
+        let (spend, adaptor, proof) = key_gen.generate_proof()?;
 
-        Ok(BobParameters {
-            buy: wallet.get_pubkey(ArbitratingKeyId::Buy)?,
-            cancel: wallet.get_pubkey(ArbitratingKeyId::Cancel)?,
-            refund: wallet.get_pubkey(ArbitratingKeyId::Refund)?,
-            adaptor,
-            extra_arbitrating_keys: extra_arbitrating_keys?,
-            arbitrating_shared_keys: arbitrating_shared_keys?,
-            spend,
-            extra_accordant_keys: extra_accordant_keys?,
-            accordant_shared_keys: accordant_shared_keys?,
-            refund_address: self.refund_address.clone(),
-            proof,
-            cancel_timelock: Some(public_offer.offer.cancel_timelock),
-            punish_timelock: Some(public_offer.offer.punish_timelock),
-            fee_strategy: Some(public_offer.offer.fee_strategy.clone()),
-        })
+        Ok((
+            BobParameters {
+                buy: key_gen.get_pubkey(ArbitratingKeyId::Buy)?,
+                cancel: key_gen.get_pubkey(ArbitratingKeyId::Cancel)?,
+                refund: key_gen.get_pubkey(ArbitratingKeyId::Refund)?,
+                adaptor,
+                extra_arbitrating_keys: extra_arbitrating_keys?,
+                arbitrating_shared_keys: arbitrating_shared_keys?,
+                spend,
+                extra_accordant_keys: extra_accordant_keys?,
+                accordant_shared_keys: accordant_shared_keys?,
+                refund_address: self.refund_address.clone(),
+                cancel_timelock: Some(public_offer.offer.cancel_timelock),
+                punish_timelock: Some(public_offer.offer.punish_timelock),
+                fee_strategy: Some(public_offer.offer.fee_strategy.clone()),
+            },
+            Proof { proof },
+        ))
     }
 
     /// Initialize the core arbitrating transactions composed of: [`Lockable`], [`Cancelable`], and
@@ -897,10 +908,10 @@ impl<Ctx: Swap> Bob<Ctx> {
         // Get the four keys, Alice and Bob for Buy and Cancel. The keys are needed, along with the
         // timelock for the cancel, to create the cancelable on-chain contract on the arbitrating
         // blockchain.
-        let alice_buy = alice_parameters.buy.clone();
-        let bob_buy = bob_parameters.buy.clone();
-        let alice_cancel = alice_parameters.cancel.clone();
-        let bob_cancel = bob_parameters.cancel.clone();
+        let alice_buy = &alice_parameters.buy;
+        let bob_buy = &bob_parameters.buy;
+        let alice_cancel = &alice_parameters.cancel;
+        let bob_cancel = &bob_parameters.cancel;
 
         // Create the data structure that represents an on-chain cancelable contract for the
         // arbitrating blockchain.
@@ -928,9 +939,9 @@ impl<Ctx: Swap> Bob<Ctx> {
         // Get the three keys, Alice and Bob for refund and Alice's punish key. The keys are
         // needed, along with the timelock for the punish, to create the punishable on-chain
         // contract on the arbitrating blockchain.
-        let alice_refund = alice_parameters.refund.clone();
-        let bob_refund = bob_parameters.refund.clone();
-        let alice_punish = alice_parameters.punish.clone();
+        let alice_refund = &alice_parameters.refund;
+        let bob_refund = &bob_parameters.refund;
+        let alice_punish = &alice_parameters.punish;
 
         // Create the data structure that represents an on-chain punishable contract for the
         // arbitrating blockchain.
@@ -955,7 +966,7 @@ impl<Ctx: Swap> Bob<Ctx> {
         let mut refund = <<Ctx::Ar as Transactions>::Refund as Refundable<
             Ctx::Ar,
             <Ctx::Ar as Transactions>::Metadata,
-        >>::initialize(&cancel, punish_lock, self.refund_address.clone())?;
+        >>::initialize(&cancel, self.refund_address.clone())?;
 
         // Set the fees according to the strategy in the offer and the local politic.
         <Ctx::Ar as Fee>::set_fee(refund.as_partial_mut(), fee_strategy, self.fee_politic)?;
@@ -988,14 +999,13 @@ impl<Ctx: Swap> Bob<Ctx> {
     ///
     pub fn cosign_arbitrating_cancel(
         &self,
-        wallet: &impl Sign<
+        wallet: &mut impl Sign<
             <Ctx::Ar as Keys>::PublicKey,
-            <Ctx::Ar as Keys>::PrivateKey,
+            <Ctx::Ar as Keys>::SecretKey,
             <Ctx::Ar as Signatures>::Message,
             <Ctx::Ar as Signatures>::Signature,
-            <Ctx::Ar as Signatures>::AdaptorSignature,
+            <Ctx::Ar as Signatures>::EncryptedSignature,
         >,
-        bob_parameters: &BobParameters<Ctx>,
         core: &CoreArbitratingTransactions<Ctx::Ar>,
     ) -> Res<CosignedArbitratingCancel<Ctx::Ar>> {
         // Extract the partial transaction from the core arbitrating bundle, this operation should
@@ -1007,8 +1017,7 @@ impl<Ctx: Swap> Bob<Ctx> {
 
         // Generate the witness message to sign and sign with the cancel key.
         let msg = cancel.generate_witness_message(ScriptPath::Failure)?;
-        let key = &bob_parameters.cancel;
-        let sig = wallet.sign_with_key(key, msg)?;
+        let sig = wallet.sign(ArbitratingKeyId::Cancel, msg)?;
 
         Ok(CosignedArbitratingCancel { cancel_sig: sig })
     }
@@ -1045,12 +1054,12 @@ impl<Ctx: Swap> Bob<Ctx> {
     ///
     pub fn validate_adaptor_refund(
         &self,
-        wallet: &impl Sign<
+        wallet: &mut impl Sign<
             <Ctx::Ar as Keys>::PublicKey,
-            <Ctx::Ar as Keys>::PrivateKey,
+            <Ctx::Ar as Keys>::SecretKey,
             <Ctx::Ar as Signatures>::Message,
             <Ctx::Ar as Signatures>::Signature,
-            <Ctx::Ar as Signatures>::AdaptorSignature,
+            <Ctx::Ar as Signatures>::EncryptedSignature,
         >,
         alice_parameters: &AliceParameters<Ctx>,
         bob_parameters: &BobParameters<Ctx>,
@@ -1066,7 +1075,7 @@ impl<Ctx: Swap> Bob<Ctx> {
 
         // Verify the adaptor refund witness
         let msg = refund.generate_witness_message(ScriptPath::Success)?;
-        wallet.verify_adaptor_signature(
+        wallet.verify_encrypted_signature(
             &alice_parameters.refund,
             &bob_parameters.adaptor,
             msg,
@@ -1117,12 +1126,12 @@ impl<Ctx: Swap> Bob<Ctx> {
     ///
     pub fn sign_adaptor_buy(
         &self,
-        wallet: &impl Sign<
+        wallet: &mut impl Sign<
             <Ctx::Ar as Keys>::PublicKey,
-            <Ctx::Ar as Keys>::PrivateKey,
+            <Ctx::Ar as Keys>::SecretKey,
             <Ctx::Ar as Signatures>::Message,
             <Ctx::Ar as Signatures>::Signature,
-            <Ctx::Ar as Signatures>::AdaptorSignature,
+            <Ctx::Ar as Signatures>::EncryptedSignature,
         >,
         alice_parameters: &AliceParameters<Ctx>,
         bob_parameters: &BobParameters<Ctx>,
@@ -1139,10 +1148,10 @@ impl<Ctx: Swap> Bob<Ctx> {
         // Get the four keys, Alice and Bob for Buy and Cancel. The keys are needed, along with the
         // timelock for the cancel, to create the cancelable on-chain contract on the arbitrating
         // blockchain.
-        let alice_buy = alice_parameters.buy.clone();
-        let bob_buy = bob_parameters.buy.clone();
-        let alice_cancel = alice_parameters.cancel.clone();
-        let bob_cancel = bob_parameters.cancel.clone();
+        let alice_buy = &alice_parameters.buy;
+        let bob_buy = &bob_parameters.buy;
+        let alice_cancel = &alice_parameters.cancel;
+        let bob_cancel = &bob_parameters.cancel;
 
         // Create the data structure that represents an on-chain cancelable contract for the
         // arbitrating blockchain.
@@ -1170,10 +1179,9 @@ impl<Ctx: Swap> Bob<Ctx> {
 
         // Generate the witness message to sign and adaptor sign with the buy key and the
         // counter-party adaptor.
-        let key = &bob_parameters.buy;
         let adaptor = &alice_parameters.adaptor;
         let msg = buy.generate_witness_message(ScriptPath::Success)?;
-        let sig = wallet.adaptor_sign_with_key(key, adaptor, msg)?;
+        let sig = wallet.encrypt_sign(ArbitratingKeyId::Buy, adaptor, msg)?;
 
         Ok(SignedAdaptorBuy {
             buy: buy.to_partial(),
@@ -1205,19 +1213,12 @@ impl<Ctx: Swap> Bob<Ctx> {
     ///
     pub fn sign_arbitrating_lock(
         &self,
-        wallet: &impl Sign<
+        wallet: &mut impl Sign<
             <Ctx::Ar as Keys>::PublicKey,
-            <Ctx::Ar as Keys>::PrivateKey,
+            <Ctx::Ar as Keys>::SecretKey,
             <Ctx::Ar as Signatures>::Message,
             <Ctx::Ar as Signatures>::Signature,
-            <Ctx::Ar as Signatures>::AdaptorSignature,
-        >,
-        key_wallet: &impl Wallet<
-            <Ctx::Ar as Keys>::PublicKey,
-            <Ctx::Ac as Keys>::PublicKey,
-            <Ctx::Ar as SharedPrivateKeys>::SharedPrivateKey,
-            <Ctx::Ac as SharedPrivateKeys>::SharedPrivateKey,
-            Ctx::Proof,
+            <Ctx::Ar as Signatures>::EncryptedSignature,
         >,
         core: &CoreArbitratingTransactions<Ctx::Ar>,
     ) -> Res<SignedArbitratingLock<Ctx::Ar>> {
@@ -1230,8 +1231,7 @@ impl<Ctx: Swap> Bob<Ctx> {
 
         // Generate the witness message to sign and sign with the fund key.
         let msg = lock.generate_witness_message(ScriptPath::Success)?;
-        let key = key_wallet.get_pubkey(ArbitratingKeyId::Fund)?;
-        let sig = wallet.sign_with_key(&key, msg)?;
+        let sig = wallet.sign(ArbitratingKeyId::Lock, msg)?;
 
         Ok(SignedArbitratingLock { lock_sig: sig })
     }
@@ -1252,14 +1252,6 @@ impl<Ctx: Swap> Bob<Ctx> {
     /// [`CoreArbitratingTransactions`] bundle is created by Bob and does not require any extra
     /// validation.
     ///
-    /// _Previously verified data_:
-    ///  * `alice_parameters`: Alice's adaptor signature in [`SignedAdaptorRefund`] bundle
-    ///  * `signed_adaptor_refund`: Verified by [`validate_adaptor_refund`]
-    ///
-    /// _Trusted data_:
-    ///  * `ar_engine`, `ac_engine`: Bob's arbitrating and accordant seeds
-    ///  * `core`: Core arbitrating transactions bundle
-    ///
     /// # Execution
     ///
     ///  * Parse the [`Refundable`] partial transaction in [`CoreArbitratingTransactions`]
@@ -1274,14 +1266,13 @@ impl<Ctx: Swap> Bob<Ctx> {
     ///
     pub fn fully_sign_refund(
         &self,
-        wallet: &impl Sign<
+        wallet: &mut impl Sign<
             <Ctx::Ar as Keys>::PublicKey,
-            <Ctx::Ar as Keys>::PrivateKey,
+            <Ctx::Ar as Keys>::SecretKey,
             <Ctx::Ar as Signatures>::Message,
             <Ctx::Ar as Signatures>::Signature,
-            <Ctx::Ar as Signatures>::AdaptorSignature,
+            <Ctx::Ar as Signatures>::EncryptedSignature,
         >,
-        bob_parameters: &BobParameters<Ctx>,
         core: CoreArbitratingTransactions<Ctx::Ar>,
         signed_adaptor_refund: &SignedAdaptorRefund<Ctx::Ar>,
     ) -> Res<FullySignedRefund<Ctx::Ar>> {
@@ -1294,12 +1285,12 @@ impl<Ctx: Swap> Bob<Ctx> {
 
         // Generate the witness message to sign and sign with the refund key.
         let msg = refund.generate_witness_message(ScriptPath::Success)?;
-        let key = &bob_parameters.refund;
-        let sig = wallet.sign_with_key(key, msg)?;
+        let sig = wallet.sign(ArbitratingKeyId::Refund, msg)?;
 
-        let key = &bob_parameters.adaptor;
-        let adapted_sig =
-            wallet.adapt_signature(key, signed_adaptor_refund.refund_adaptor_sig.clone())?;
+        let adapted_sig = wallet.decrypt_signature(
+            AccordantKeyId::Spend,
+            signed_adaptor_refund.refund_adaptor_sig.clone(),
+        )?;
 
         Ok(FullySignedRefund {
             refund_sig: sig,
@@ -1307,8 +1298,22 @@ impl<Ctx: Swap> Bob<Ctx> {
         })
     }
 
-    pub fn recover_accordant_assets(&self) -> Res<()> {
-        todo!()
+    pub fn recover_accordant_assets(
+        &self,
+        wallet: &mut impl Sign<
+            <Ctx::Ar as Keys>::PublicKey,
+            <Ctx::Ar as Keys>::SecretKey,
+            <Ctx::Ar as Signatures>::Message,
+            <Ctx::Ar as Signatures>::Signature,
+            <Ctx::Ar as Signatures>::EncryptedSignature,
+        >,
+        alice_parameters: &AliceParameters<Ctx>,
+        adaptor_buy: SignedAdaptorBuy<Ctx::Ar>,
+        buy_tx: <Ctx::Ar as Onchain>::Transaction,
+    ) -> <Ctx::Ar as Keys>::SecretKey {
+        let encryption_key = &alice_parameters.adaptor;
+        let signature = <<Ctx::Ar as Transactions>::Buy>::extract_witness(buy_tx);
+        wallet.recover_secret_key(adaptor_buy.buy_adaptor_sig, encryption_key, signature)
     }
 }
 
@@ -1323,12 +1328,18 @@ pub trait Arbitrating:
     + Signatures
     + Timelock
     + Transactions
-    + SharedPrivateKeys
+    + SharedSecretKeys
     + Clone
     + Eq
 {
 }
 
 /// An accordant is the blockchain which does not need transaction inside the protocol nor
-/// timelocks, it is the blockchain with the less requirements for an atomic swap.
-pub trait Accordant: Asset + Address + Keys + SharedPrivateKeys + Clone + Eq {}
+/// timelocks: it is the blockchain with fewer requirements for an atomic swap.
+pub trait Accordant: Asset + Address + Keys + SharedSecretKeys + Clone + Eq {
+    /// Derive the lock address for the accordant blockchain.
+    fn derive_lock_address(
+        network: Network,
+        keys: SwapAccordantKeys<Self>,
+    ) -> Result<Self::Address, crypto::Error>;
+}
